#include	<stdio.h>
#include	<stdlib.h>
#include	<fcntl.h>
#include	<machine/cpufunc.h>

typedef	unsigned char	BYTE;
typedef	unsigned short	WORD;

#define	FAILURE	(-1)

#define	TGDC_STAT_REG	0x60	/* read */
#define	TGDC_PARAM_REG	0x60	/* write */
#define	TGDC_DATA_REG	0x62	/* read */
#define	TGDC_CMD_REG	0x62	/* write */
#define	MODE_REG1		0x68

#ifndef	GDC_START
#define	GDC_START	0x0D
#define	GDC_STOP	0x0C
#endif

/* text GDC primitives */

void	PC98WaitTGDCReady	(void)
{
	while (inb (TGDC_STAT_REG) & 2);
}

void	PC98StartText	(void)
{
	PC98WaitTGDCReady ();
	outb (TGDC_CMD_REG, GDC_START);
}

void	PC98StopText	(void)
{
	PC98WaitTGDCReady ();
	outb (TGDC_CMD_REG, GDC_STOP);
}

void	PC98SetKCGCodeAccess	(void)
{
	outb (MODE_REG1, 0x0A);
}

void	PC98SetKCGDotAccess	(void)
{
	outb (MODE_REG1, 0x0B);
}

void	PC98RegisterGaijiWithIO	(BYTE *buf)
{
	int	y;

	outb (0xA1, *buf++);
	outb (0xA3, *buf++ - 0x20);

	for (y = 0; y < 16; y++)
	{
		outb (0xA5, y | 0x20);
		outb (0xA9, *buf++);
		outb (0xA5, y);
		outb (0xA9, *buf++);
	}
}

#define	DEFAULT_DIR	"/usr/share/syscons/fonts"
#define	DEFAULT_FONT_FILE	"uskcg16.sys"
#define	DEV_IO	"/dev/io"
#define	REC_SIZE	34

void	main	(int argc, char **argv)
{
	int	fd, fdio, i, n, j;
	BYTE	gaijibuf [REC_SIZE];
	char	buf [FILENAME_MAX], filename [FILENAME_MAX];

	if (argc == 1)
	{
		fprintf (stderr, "usage: %s filename...\n", argv [0]);
		exit (EXIT_FAILURE);
	}

	if ((fdio = open (DEV_IO, O_RDWR)) == FAILURE)
	{
		sprintf (buf, "%s: " DEV_IO, argv [0]);
		perror (buf);
		exit (EXIT_FAILURE);
	}
	
	for (j = 1; argv [j] != NULL; j++)
	{
		if ((fd = open (argv [j], O_RDONLY)) == FAILURE)
		{
			sprintf (filename, DEFAULT_DIR "/%s", argv [j]);
			if ((fd = open (filename, O_RDONLY)) == FAILURE)
			{
				sprintf (buf, "%s: %s", argv [0], filename);
				perror (buf);
				close (fdio);
				exit (EXIT_FAILURE);
			}
		}

		PC98StopText ();
		PC98SetKCGDotAccess ();
		read (fd, gaijibuf, REC_SIZE);
		n = *(WORD *)gaijibuf;
		for (i = 0; i < n; i++)
		{
			if (read (fd, gaijibuf, REC_SIZE) < REC_SIZE) break;
			PC98RegisterGaijiWithIO (gaijibuf);
		}
		PC98SetKCGCodeAccess ();
		PC98StartText ();
		printf ("%s: %d characters registered.\n", argv[j], i);
		close (fd);
	}

	close (fdio);
}
